local BPM  = 50.;
local BEAT = 1000./(BPM/60);

local FONT = "res/font/shippori.ttf";

function B(b)
  return b*BEAT;
end

--
-- Adapted from
-- Tweener's easing functions (Penner's Easing Equations)
-- and http://code.google.com/p/tweener/ (jstweener javascript version)
--
local function inQuint(t)
  t = t;
  return math.pow(t, 5);
end
local function outQuint(t)
  t = t - 1;
  return math.pow(t, 5) + 1;
end
local function outInQuint(t)
  if t < .5 then
    return .5*outQuint(t * 2);
  else
    return .5*inQuint(t * 2 - 1) + .5;
  end
end
local function inCirc(t)
  return -(math.sqrt(1 - math.pow(t, 2)) - 1);
end
local function outCirc(t)
  t = t - 1;
  return math.sqrt(1 - math.pow(t, 2));
end
local function outInCirc(t)
  if t < .5 then
    return .5*outCirc(t * 2);
  else
    return .5*inCirc((t * 2) - 1) + .5;
  end
end


Music(0, 60*BEAT, "res/music/kimigayo.mp3", 0, function(t, obj)
  t = t*60/4;
  obj.volume = math.min(t, 1);
  obj.lpf    = 1-obj.volume;
end);
local b = 0;

-- b8 rest
Glyph(B(b+0), B(b+8), "国歌斉唱", FONT, 64, function (t, obj)
  obj.alpha = 1-math.abs(t*2-1);
  obj.posY  = .3;
end);
b = b+8;


-- b8 君が代は
Glyph(B(b+0), B(b+8), "君が代は", FONT, 64, function (t, obj)
  obj.posY  = -(t*2-1)*1.8;
  obj.alpha = .5;
end);

InputWin(B(b+1), B(b+8), "君が代は", "きみがよは", function () end);

b = b+8;


-- b8 千代に 八千代に
Glyph(B(b+1), B(b+5), "千代に", FONT, 64, function (t, obj)
  obj.posX  = (outInQuint(t)*2-1)*1.8;
  obj.posY  = .3;
  obj.alpha = (1-t)*.7+.3;
end);
Glyph(B(b+4), B(b+8), "八千代に", FONT, 64, function (t, obj)
  obj.posX  = -(outInQuint(t)*2-1)*1.8;
  obj.posY  = .3;
  obj.alpha = (1-t)*.7+.3;
end);

InputWin(B(b+1), B(b+4), "千代に", "ちよに", function () end);
InputWin(B(b+4), B(b+8), "八千代に", "やちよに", function () end);

b = b+8;


-- b8 さざれ 石の
Glyph(B(b+0), B(b+5), "さざれ", FONT, 64, function (t, obj)
  obj.posY  = (t*2-1)*1.7;
  obj.alpha = .5;
end);
Glyph(B(b+4), B(b+7), "石の", FONT, 64, function (t, obj)
  local s = math.modf(t*3)/3;
  obj.posY  = .3;
  obj.alpha = s*.7+.3;
  obj.scaleX = s*.6 + .7;
  obj.scaleY = s*.6 + .7;
end);

InputWin(B(b+0), B(b+4), "さざれ", "さざれ", function () end);
InputWin(B(b+4), B(b+8), "石の", "いしの", function () end);

b = b+8;


-- b8 巌となりて
Glyph(B(b+.5), B(b+8), "巌", FONT, 64, function (t, obj)
  local i = math.modf(t*7.5)

  obj.posY = .3;
  if i == 0 then
    obj.scaleX = .5;
    obj.scaleY = .8;
    obj.alpha = .5;
  elseif i == 1 then
    obj.scaleX = 1;
    obj.scaleY = .5;
    obj.alpha = .4;
  elseif i == 2 then
    obj.scaleX = .8;
    obj.scaleY = 1.6;
    obj.alpha = .6;
  elseif i == 3 then
    obj.scaleX = 1;
    obj.scaleY = 1.3;
    obj.alpha = .8;
  else
    obj.scaleX = 1.3;
    obj.scaleY = 1.6;
    obj.alpha = .4;
  end
end);
Glyph(B(b+4.5), B(b+8), "巌と成りて", FONT, 64, function (t, obj)
  obj.posY  = .3;
  obj.alpha = .6;
end);

InputWin(B(b+0), B(b+8), "巌と成りて", "いわおとなりて", function () end);

b = b+8;


-- b16 苔の生すまで
Glyph(B(b+1), B(b+10), "苔の生す", FONT, 64, function (t, obj)
  obj.posY  = .3;
  obj.alpha = (1-t)*.9+.1;
end);
Glyph(B(b+9), B(b+16), "ま", FONT, 64, function (t, obj)
  local a = 1-outInCirc(t)*2;
  obj.posY    = .3;
  obj.posX    = a*1.8-.6;
  obj.scaleX  = .8;
  obj.rota    = a*math.pi*2*.1;
  obj.alpha   = 1-math.pow(t, 2);
end);
Glyph(B(b+9), B(b+16), "で", FONT, 64, function (t, obj)
  local a = 1-outInCirc(t)*2;
  obj.posY    = .3;
  obj.posX    = -a*1.8+.6;
  obj.scaleX  = .6;
  obj.rota    = -a*math.pi*2*.1;
  obj.alpha   = 1-math.pow(t, 2);
end);

InputWin(B(b+1), B(b+16), "苔の生すまで", "こけのむすまで", function () end);

b = b+16;


-- b8 rest
Glyph(B(b+0), B(b+2), "日本国歌", FONT, 64, function (t, obj)
  obj.alpha = math.min(t*2, 1);
  obj.posY  = .3;
end);
Glyph(B(b+2), B(b+6), "君が代", FONT, 64, function (t, obj)
  obj.alpha = 1-t;
  obj.posY  = .3;
end);
b = b+8;